.imger-wr{
	position: fixed;
	top: 0px;
	left: 0px;
	width: 100%;
	height: 100%;
	background-color: rgba(0, 0, 0, 0.8);
}

/* Change noUISlider default css */

.imger-wr .noUi-horizontal{
	height: 8px;
	border: 1px solid #515050;
}

.imger-wr .noUi-horizontal .noUi-handle{
	width: 28px;
	top: -10px;
	border-radius: 50%;
	border: none;
	box-shadow: 0px 0px 1px rgb(123, 123, 123) inset, 0px 1px 7px rgb(180, 180, 180) inset, 0px 3px 6px -3px rgb(0, 0, 0);
}

.imger-wr .noUi-horizontal .noUi-handle:before{
	background: none;
}
.imger-wr .noUi-horizontal .noUi-handle:after{
	background: none;
}

.imger-wr .noUi-background{
	background: none;
	box-shadow: none;
}

.imger-wr .noUi-target{
	border: 1px solid #515050;
}

.imger-wr .noUi-connect{
	background: none repeat scroll 0% 0% #454545;
}

/* END Change noUISlider default css */

.imger-window {
	position: absolute;
	top: 10px;
	left: 10px;
	right: 10px;
	bottom: 10px;
	background-color: white;
	border-radius: 5px;
}

.imger-display{
	position: absolute;
	top: 5px;
	left: 5px;
	right: 5px;
	bottom: 85px;
	background-color: #252525;
	text-align: center;
	overflow: auto;
}

.imger-infobar{
	position: absolute;
	height: 40px;
	left: 5px;
	right: 5px;
	bottom: 45px;
	background-color: #333;
	text-align: center;
}

.imger-toolbar-wr{
	position: absolute;
	height: 40px;
	left: 5px;
	right: 5px;
	bottom: 5px;
}

.imger-toolbar-tbl{
	position: relative;
	width: 100%;
}

.imger-toolbar-auto-cell{
	width:1%;
	white-space:nowrap;
}

.imger-toolbar-auto-cell button{
	margin-left: 5px;
}

.imger-toolbar-row{
	vertical-align: middle;
}

.imger-logo{
	position: relative;
	display: inline-block;
	height: 40px;
	line-height: 40px;
	font-family: Arial;
	font-weight: bold;
	margin-left: 10px;
}

.imger-logo:after{
	content: 'Imger';
}

.imger-scaler-wr{
	position: relative;
	width: 300px;
	display: inline-block;
	padding-top: 14px;
}

.imger-user-info{
	padding-top: 20px;
	padding-bottom: 20px;
	color: #929292;
	text-align: center;
}

.imger-canvas{
	position: relative;
	width: 300px;
	height: 200px;
	box-shadow: 0px 0px 5px rgba(0, 0, 0, 0.3);
}

.imger-image-info{
	position: absolute;
	left: 0px;
	top: 0px;
	line-height: 40px;
	color: #CCC;
	padding-left: 10px;
}

.imger-img-url-wr{
	display: inline-block;
	width: 300px;
	vertical-align: middle;
}

.imger-form{
	position: absolute;
	top: 15px;
	right: 15px;
	bottom: 55px;
	padding: 0px;
	min-width: 300px;
	display: none;
	background-color: rgba(255, 255, 255, 0.8);
	overflow: auto;
}

.imger-form-wr{
	padding-left: 10px;
	padding-right: 10px;
}

.imger-form-wr .input-group{
	width: 100%;
}

.imger-form-btn-wr{
	padding: 10px;
}

.imger-form-note{
	padding: 10px;
}

.canvas-wr{
	position: relative;
	display: inline-block;
}

/* THE BELOW SHOUD PROBABLY BE SEPARATED */

.input-box{
	border: 1px solid #A8A8A8;
	margin-bottom: 5px;
	padding: 5px;
}

.imger-form-wr textarea{
	position: relative;
	width: 100%;
	min-height: 100px;
	margin-bottom: 5px;
}

.imger-form-wr select{
	position: relative;
	width: 100%;
	margin-bottom: 5px;
}

.imger-form-wr .input-group{
	margin-bottom: 5px;
}
