<div class="imger-wr">
	<div class="imger-window">
		<div class="imger-display">
			<div class="imger-user-info"></div>
			<div class="canvas-wr">
				<canvas class="imger-canvas" id="imger-canvas"></canvas>
			</div>
		</div>
		<div class="imger-infobar">
			<span class="imger-image-info"></span>
			<div class="imger-scaler-wr">
	            <div class="imger-scaler"></div>
	        </div>
		</div>
		<div class="imger-toolbar-wr">
			<table class="imger-toolbar-tbl">
				<tbody>
					<tr class="imger-toolbar-row">
						<td class="imger-toolbar-auto-cell">
							<a href="https://github.com/4shaw/imger" target="_blank"><div class="imger-logo"></div></a>
						</td>
						<td></td>
						<td class="imger-toolbar-auto-cell">
							<div class="imger-img-url-wr">
								<div class="input-group">
									<input type="text" id="getURLInput" class="form-control" placeholder="Image URL" />
									<span class="input-group-btn">
										<button class="btn btn-default get-URL" type="button">
											GET
										</button>
									</span>
								</div>
							</div>
							<button class="btn btn-default imger-btn-browse">
								<!--span class="glyphicon glyphicon-folder-open" style="margin-right:5px;"></span-->
								Browse
							</button>
							<button class="btn btn-info imger-btn-done">Done</button>
							<button class="btn btn-default imger-btn-cancel">
								<span class="glyphicon glyphicon-remove"></span>
							</button>
						</td>
					</tr>
				</tbody>
			</table>
		</div>
	</div>
	<div class="imger-form">
		<div class="imger-form-note"></div>
		<div class="imger-form-wr"></div>
		<div class="imger-form-btn-wr">
			<button class="btn btn-info imger-form-sbmt-btn">Submit</button>
			<button class="btn btn-default imger-form-cancel-btn">
				<span class="glyphicon glyphicon-remove"></span>
			</button>
		</div>
	</div>
</div>
